import logging
import logging.config
from pprint import pformat
from typing import Optional, List
from os import environ
from pathlib import Path

from rsyncrun.log_setup import log_config
from rich.logging import RichHandler

LOG_LEVELS = ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]

logger = logging.getLogger("rsync_runner")


class Config(dict):
    """
    A versatile config_json object that can parse many file types, load environment variables.
    Multiple config_json objects from different unique sources can be collated easily with a precedence value
    (higher precedence values trump lower precedence values).
    """
    log_level: Optional[str] = "DEBUG"
    log_config: Optional[dict] = log_config
    forbidden_attrs: List[str] = [""]

    def __init__(
            self,
            env_prefix: Optional[str] = None,
            **kwargs
    ):
        self.log_directory_path = None
        if env_prefix is not None and isinstance(env_prefix, str):
            self.prefix = env_prefix
            data = {k: v for k, v in environ.items() if env_prefix in k}
        elif isinstance(env_prefix, str) and env_prefix == "*":
            data = environ
        else:
            data = {}

        data.update(**kwargs)
        pretty_data = pformat(data, indent=2)
        logger.debug(f"{self}\n{pretty_data}")
        self.populate_attrs(data)

        super().__init__(**data)

    def populate_attrs(self, data: dict):
        for attr, val in data.items():
            if isinstance(val, Path):
                val = val.resolve()
            setattr(self, attr, val)

    @classmethod
    def data_without_metadata(cls, instance) -> dict:
        if not isinstance(instance,  cls):
            raise AttributeError(
                f"Unable to parse data from object {instance} as it "
                f"is not an instance or subclass instance of {cls}"
            )

        return {k: v for k, v in instance.__dict__.items() if k not in cls.forbidden_attrs}

    def setup_logging(self):
        log_path: str = str(self.log_path) if hasattr(self, "log_path") and getattr(self, "log_path") else None
        log_level: str = self.log_level if hasattr(self, "log_level") and getattr(self, "log_level") else "INFO"
        log_config['handlers']['file']['level'] = log_level
        log_config['handlers']['file']['filename'] = log_path
        log_config['loggers']['rsync_runner']['level'] = log_level
        if log_path == None:
            log_config['handlers'] = {k: v for k, v in log_config.get('handlers').items() if k != 'file'}
            log_config['loggers']['rsync_runner']['handlers'] = ['custom']
        logging.config.dictConfig(log_config)

        logger = logging.getLogger("rsync_runner")
        formatter = logging.Formatter(
            fmt="[bold][italic][#dedede]%(filename)s.%(funcName)s:%(lineno)d\t\t[/bold][/italic][/#dedede] %(message)s",
            datefmt="[%d/%M/%Y %H:%M:%S] "
        )
        for handler in logger.handlers:
            if handler.name == "custom":
                handler.formatter = formatter
                setattr(handler, "markup", True)
        if log_path:
            self.log_directory_path = Path(log_path).parent